#region Copyright Syncfusion Inc. 2001 - 2005
//
//  Copyright Syncfusion Inc. 2001 - 2005. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.IO;
using System.Drawing;
using System.Collections;

using Syncfusion.Calculate;
using Syncfusion.XlsIO;

namespace XlsFileUsingExcelRW
{
	/// <summary>
	/// A CalcWorkbook-derived object that uses ExcelRW to read / compute Excel XLS files.
	/// </summary>
	public class ExcelRWCalcWorkbook : CalcWorkbook
	{
		/// <summary>
		/// The ExcelRW IWorkbook used by this class.
		/// </summary>
		public IWorkbook excelRWWB;
		 
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="calcSheets">Array of ExcelRWCalcSheet objects used in this workbook.</param>
		/// <param name="namedRanges">A hashtable containing named ranges.</param>
		public ExcelRWCalcWorkbook(ExcelRWCalcSheet[]calcSheets, Hashtable namedRanges) 
			: base(calcSheets, namedRanges)
		{

		}

		/// <summary>
		/// Creates a ExceRWCalcWorkbook object from an XLS file.
		/// </summary>
		/// <param name="fileName">Complete pathname of the XLS file.</param>
		/// <returns></returns>
		public static ExcelRWCalcWorkbook CreateFromXLS(string fileName)
		{
			IWorkbook wb;
			try
			{
				wb = ExcelUtils.Open(fileName);
				ExcelUtils.ThrowNotSavedOnDestroy = false;
			}
			catch
			{
				throw new FileLoadException("XlsIO cannot load the file.", fileName);
			}

			ExcelRWCalcSheet[] sheets = new ExcelRWCalcSheet[wb.Worksheets.Count];

			for(int i = 0; i < wb.Worksheets.Count; ++i)
			{
				sheets[i] = new ExcelRWCalcSheet(wb.Worksheets[i]);
				sheets[i].Name = wb.Worksheets[i].Name;
				
			}

			Hashtable ranges = new Hashtable();
			foreach(IName name in wb.Names)
			{
				ranges.Add(name.Name.ToUpper(), name.Value.Replace("'", ""));
			}

			ExcelRWCalcWorkbook cwb = new ExcelRWCalcWorkbook(sheets, ranges);
			cwb.excelRWWB = wb;
			return cwb;
		}

		/// <summary>
		/// Performs all calculations in the workbook.
		/// </summary>
		public override void CalculateAll()
		{
			foreach(CalcSheet sheet in this.CalcSheetList)
			{
				sheet.CalculationsSuspended = false;
			}

			//DateTime start = DateTime.Now;
			//DateTime loopStart = DateTime.Now;
			foreach(ExcelRWCalcSheet sheet in this.CalcSheetList)
			{
				if(sheet != null)
				{
					for(int row = 1; row <= sheet.RowCount; ++row)
					{
						for(int col = 1; col <= sheet.ColCount; ++col)
						{
							object o = sheet[row, col];
							if(o != null)
							{
								string s2 = o.ToString();
								if(s2.Length > 0 && s2[0] == '=')
								{
									Point key = new Point(row, col);
									if(sheet.formulasCashe.ContainsKey(key))
									{
										sheet.formulasCashe[key] = s2;
									}
									else
									{
										sheet.formulasCashe.Add(key, s2);
									}
									sheet[row, col] = s2;
								}
							}
						}
					
					}
				}
				//Console.WriteLine(string.Format("{0}  {2}  parsed {1}",  DateTime.Now - start , sheet.Name, DateTime.Now - loopStart));
				//loopStart = DateTime.Now;
				//this.label5.Text = string.Format("Sheets {0}", loop++);
				//Application.DoEvents();
			}
		}
	}

	/// <summary>
	/// Specially derived CalcSheet for use with ExcelRW.
	/// </summary>
	public class ExcelRWCalcSheet : CalcSheet
	{
		public Hashtable formulasCashe = new Hashtable();
		
		public IWorksheet excelSheet;

		/// <summary>
		/// Constructor
		/// </summary>
		public ExcelRWCalcSheet()
		{
			excelSheet = null;
		}

		/// <summary>
		/// Constuctor
		/// </summary>
		/// <param name="sheet">Underlying ExcelRw IWorksheet object.</param>
		public ExcelRWCalcSheet(IWorksheet sheet)
		{
			excelSheet = sheet;
		}

		/// <summary>
		/// Sets a value into the ExcelRW worksheet.
		/// </summary>
		/// <param name="rowPos">The row index of the changed value.</param>
		/// <param name="colPos">The column index of the changed value.</param>
		/// <param name="val">The new value.</param>
		/// <remarks>
		/// If CalculationsSuspended is not True, a ValueChanged event is raised.
		/// </remarks>
		public override void SetValue(int rowPos, int colPos, string val)
		{

			SetValueRowCol(val, rowPos, colPos);
			if(CalculationsSuspended)
				return;

			
			ValueChangedEventArgs e1 = new ValueChangedEventArgs(rowPos, colPos, val);
			base.OnValueChanged(e1);
		}

		/// <summary>
		/// Number of rows in this spreadsheet.
		/// </summary>
		public new int RowCount
		{

			get{return this.excelSheet.UsedRange.Rows.GetLength(0);}
		}

		/// <summary>
		/// The number of columns in this spreadsheet.
		/// </summary>
		public new int ColCount
		{
			get{return this.excelSheet.UsedRange.Columns.GetLength(0);}
		}

		 
		/// <summary>
		/// Returns the value at the specified row and column.
		/// </summary>
		/// <param name="row">Row index (one-based).</param>
		/// <param name="col">Column index (one-based).</param>
		/// <returns>The value at the specified row and column.</returns>
		public override object GetValueRowCol(int row, int col)
		{
			object o = excelSheet[row, col].Formula;
			if(o == null)
				o = excelSheet[row, col].Value;
			if(o != null)
			{
				return o.ToString();//.Replace("'", ""); //keep the tic defect#541
			}
			return o;
		}

		/// <summary>
		/// Set the value at a specified row and column.
		/// </summary>
		/// <param name="value">The value to be set.</param>
		/// <param name="rowPos">The row index (one-based).</param>
		/// <param name="colPos">The column index (one-based).</param>
		public override void SetValueRowCol(object value, int row, int col)
		{
			//excelSheet[rowPos + 1, colPos + 1].Value = value.ToString();
			excelSheet[row, col].Value = value.ToString();
		}
	}

}
